@@ -2,7 +2,7 @@
  * Seahorse
  *
  * Copyright (C) 2003 Jacob Perkins
- * Copyright (C) 2004 Nate Nielsen
+ * Copyright (C) 2004 Stefan Walter
  * Copyright (C) 2005 Adam Schreiber
  *
  * This program is free software; you can redistribute it and/or modify
@@ -530,98 +530,108 @@ seahorse_pgp_key_op_sign (SeahorsePGPKey *pkey, SeahorsePGPKey *signer,
 }
 
 typedef enum {
-	PASS_START,
-	PASS_COMMAND,
+    PASS_START,
+    PASS_COMMAND,
     PASS_PASSPHRASE,
-	PASS_QUIT,
-	PASS_SAVE,
-	PASS_ERROR
+    PASS_QUIT,
+    PASS_SAVE,
+    PASS_ERROR
 } PassState;
 
 /* action helper for changing passphrase */
 static gpgme_error_t
 edit_pass_action (guint state, gpointer data, int fd)
 {
-	switch (state) {
-		case PASS_COMMAND:
-            PRINT ((fd, "passwd"));
-			break;
-        case PASS_PASSPHRASE:
-            /* Do nothing */
-            return GPG_OK;
-		case PASS_QUIT:
-            PRINT ((fd, QUIT));
-			break;
-		case PASS_SAVE:
-            PRINT ((fd, YES));
-			break;
-		default:
-			return GPG_E (GPG_ERR_GENERAL);
-	}
-	
+    switch (state) {
+    case PASS_COMMAND:
+        PRINT ((fd, "passwd"));
+        break;
+    case PASS_PASSPHRASE:
+        /* Do nothing */
+        return GPG_OK;
+    case PASS_QUIT:
+        PRINT ((fd, QUIT));
+        break;
+    case PASS_SAVE:
+        PRINT ((fd, YES));
+        break;
+    default:
+        return GPG_E (GPG_ERR_GENERAL);
+    }
+    
     PRINT ((fd, "\n"));
-	return GPG_OK;
+    return GPG_OK;
 }
 
 /* transition helper for changing passphrase */
 static guint
 edit_pass_transit (guint current_state, gpgme_status_code_t status,
-		   const gchar *args, gpointer data, gpgme_error_t *err)
+                   const gchar *args, gpointer data, gpgme_error_t *err)
 {
-	guint next_state;
-	
-	switch (current_state) {
-		/* start state, go to command */
-		case PASS_START:
-			if (status == GPGME_STATUS_GET_LINE && g_str_equal (args, PROMPT))
-				next_state = PASS_COMMAND;
-			else {
-                *err = GPG_E (GPG_ERR_GENERAL);
-                g_return_val_if_reached (PASS_ERROR);
-			}
-			break;
-		/* did command, go to should be the passphrase now */
-		case PASS_COMMAND:
-            if (status == GPGME_STATUS_NEED_PASSPHRASE_SYM)
-    			next_state = PASS_PASSPHRASE;
-            else {
-                *err = GPG_E (GPG_ERR_GENERAL);
-                g_return_val_if_reached (PASS_ERROR);
-          }
-          break;
-        /* got passphrase now quit */
-        case PASS_PASSPHRASE:
-            if (status == GPGME_STATUS_GET_LINE && g_str_equal (args, PROMPT))
-             next_state = PASS_QUIT;
-         else {
-                *err = GPG_E (GPG_ERR_GENERAL);
-                g_return_val_if_reached (PASS_ERROR);
-           }
-          break;
-        
-		/* quit, go to save */
-		case PASS_QUIT:
-			if (status == GPGME_STATUS_GET_BOOL && g_str_equal (args, SAVE))
-				next_state = PASS_SAVE;
-			else {
-                *err = GPG_E (GPG_ERR_GENERAL);
-                g_return_val_if_reached (PASS_ERROR);
-			}
-			break;
-		/* error, go to quit */
-		case PASS_ERROR:
-			if (status == GPGME_STATUS_GET_LINE && g_str_equal (args, PROMPT))
-				next_state = PASS_QUIT;
-			else
-				next_state = PASS_ERROR;
-			break;
-		default:
+    guint next_state;
+    
+    switch (current_state) {
+    /* start state, go to command */
+    case PASS_START:
+        if (status == GPGME_STATUS_GET_LINE && g_str_equal (args, PROMPT))
+            next_state = PASS_COMMAND;
+        else {
             *err = GPG_E (GPG_ERR_GENERAL);
             g_return_val_if_reached (PASS_ERROR);
-			break;
-	}
-	
-	return next_state;
+        }
+        break;
+
+    case PASS_COMMAND:
+        /* did command, go to should be the passphrase now */
+        if (status == GPGME_STATUS_NEED_PASSPHRASE_SYM)
+            next_state = PASS_PASSPHRASE;
+        
+        /* If all tries for passphrase were wrong, we get here */
+        else if (status == GPGME_STATUS_GET_LINE && g_str_equal (args, PROMPT)) {
+            *err = GPG_E (GPG_ERR_CANCELED);
+            next_state = PASS_ERROR;
+        
+        /* No idea how we got here ... */
+        } else {
+            *err = GPG_E (GPG_ERR_GENERAL);
+            g_return_val_if_reached (PASS_ERROR);
+        }
+        break;
+    /* got passphrase now quit */
+    case PASS_PASSPHRASE:
+        if (status == GPGME_STATUS_GET_LINE && g_str_equal (args, PROMPT))
+            next_state = PASS_QUIT;
+        else {
+            *err = GPG_E (GPG_ERR_GENERAL);
+            g_return_val_if_reached (PASS_ERROR);
+        }
+        break;
+        
+    /* quit, go to save */
+    case PASS_QUIT:
+        if (status == GPGME_STATUS_GET_BOOL && g_str_equal (args, SAVE))
+            next_state = PASS_SAVE;
+        else {
+            *err = GPG_E (GPG_ERR_GENERAL);
+            g_return_val_if_reached (PASS_ERROR);
+        }
+        break;
+    
+    /* error, go to quit */
+    case PASS_ERROR:    
+        if (status == GPGME_STATUS_GET_LINE && g_str_equal (args, PROMPT))
+            next_state = PASS_QUIT;
+        else
+            next_state = PASS_ERROR;
+        break;
+    
+    default:
+        *err = GPG_E (GPG_ERR_GENERAL);
+        g_return_val_if_reached (PASS_ERROR);
+        break;
+    }
+    
+    return next_state;
 }
 
 /**
@@ -1424,35 +1434,22 @@ add_key_transit (guint current_state, gpgme_status_code_t status,
 			break;
 		/* did command, do type */
 		case ADD_KEY_COMMAND:
+		case ADD_KEY_TYPE:
+		case ADD_KEY_LENGTH:
+		case ADD_KEY_EXPIRES:
 			if (status == GPGME_STATUS_GET_LINE && g_str_equal (args, "keygen.algo"))
 				next_state = ADD_KEY_TYPE;
-			else {
-                *err = GPG_E (GPG_ERR_GENERAL);
-                g_return_val_if_reached (ADD_KEY_ERROR);
-			}
-			break;
-		/* did type, do length */
-		case ADD_KEY_TYPE:
-			if (status == GPGME_STATUS_GET_LINE && g_str_equal (args, "keygen.size"))
+			else if (status == GPGME_STATUS_GET_LINE && g_str_equal (args, "keygen.size"))
 				next_state = ADD_KEY_LENGTH;
-			else {
-                *err = GPG_E (GPG_ERR_GENERAL);
-                g_return_val_if_reached (ADD_KEY_ERROR);
-			}
-			break;
-		/* did length, do expires */
-		case ADD_KEY_LENGTH:
-			if (status == GPGME_STATUS_GET_LINE && g_str_equal (args, "keygen.valid"))
+			else if (status == GPGME_STATUS_GET_LINE && g_str_equal (args, "keygen.valid"))
 				next_state = ADD_KEY_EXPIRES;
+			else if (status == GPGME_STATUS_GET_LINE && g_str_equal (args, PROMPT))
+				next_state = ADD_KEY_QUIT;
 			else {
                 *err = GPG_E (GPG_ERR_GENERAL);
-                g_return_val_if_reached (ADD_KEY_ERROR);
+                return ADD_KEY_ERROR; /* Legitimate errors error here */
 			}
 			break;
-		/* did expires, quit */
-		case ADD_KEY_EXPIRES:
-			next_state = ADD_KEY_QUIT;
-			break;
 		/* quit, save */
 		case ADD_KEY_QUIT:
 			if (status == GPGME_STATUS_GET_BOOL && g_str_equal (args, SAVE))
